﻿using IRaCIS.Core.API;
using IRaCIS.Core.Application.BusinessFilter;
using IRaCIS.Core.Application.Filter;
using IRaCIS.Core.Application.MassTransit.Consumer;
using IRaCIS.Core.Application.Service.BackGroundJob;
using IRaCIS.Core.Infra.EFCore;
using IRaCIS.Core.Infrastructure.Extention;
using LogDashboard;
using MassTransit;
using MassTransit.NewIdProviders;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Newtonsoft.Json;
using Serilog;
using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Runtime.InteropServices;

AppContext.SetSwitch("Npgsql.EnableLegacyTimestampBehavior", true);
AppContext.SetSwitch("Npgsql.DisableDateTimeInfinityConversions", true);

#region 获取环境变量
//以配置文件为准，否则 从url中取环境值(服务以命令行传递参数启动，配置文件配置了就不需要传递环境参数)
var config = new ConfigurationBuilder()
            .AddEnvironmentVariables()
            .Build();

var enviromentName = config["ASPNETCORE_ENVIRONMENT"];

if (string.IsNullOrWhiteSpace(enviromentName))
{

    var index = Array.IndexOf(args, "--env");
    enviromentName = index > -1
        ? args[index + 1]
        : "Development";
}
#endregion

var builder = WebApplication.CreateBuilder(new WebApplicationOptions
{
    EnvironmentName = enviromentName
});

#region 主机配置


NewId.SetProcessIdProvider(new CurrentProcessIdProvider());

builder.Configuration.AddJsonFile(ConfigMapFileProvider.FromRelativePath(""), "appsettings.json", false, true)
                     .AddJsonFile(ConfigMapFileProvider.FromRelativePath(""), $"appsettings.{enviromentName}.json", false, true);

builder.Host.UseSerilog();


#region Autofac 废弃
//builder.Host.UseServiceProviderFactory(new AutofacServiceProviderFactory())
//            .ConfigureContainer<ContainerBuilder>(containerBuilder =>
//            {
//                containerBuilder.RegisterModule<AutofacModuleSetup>();
//            }).UseWindowsService();
#endregion

#endregion


#region 配置服务
var _configuration = builder.Configuration;

//手动注册服务
builder.Services.ConfigureServices(_configuration);

//异常处理
//builder.Services.AddExceptionHandler<GlobalExceptionHandler>();
//builder.Services.AddProblemDetails();

//健康检查
builder.Services.AddHealthChecks();
//本地化
builder.Services.AddJsonLocalization(options => options.ResourcesPath = "Resources");

// 异常、参数统一验证过滤器、Json序列化配置、字符串参数绑型统一Trim()
builder.Services.AddControllers(options =>
{
    //options.Filters.Add<LogActionFilter>();
    options.Filters.Add<ModelActionFilter>();
    options.Filters.Add<ProjectExceptionFilter>();
    options.Filters.Add<UnitOfWorkFilter>();
    //options.Filters.Add<EncreptApiResultFilter>(10);
    options.Filters.Add<LimitUserRequestAuthorization>();


})
    .AddNewtonsoftJsonSetup(builder.Services); // NewtonsoftJson 序列化  处理

//动态WebApi + UnifiedApiResultFilter  省掉控制器代码
builder.Services.AddDynamicWebApiSetup();
//AutoMapper
builder.Services.AddAutoMapperSetup();
//EF ORM  QueryWithNoLock
builder.Services.AddEFSetup(_configuration, enviromentName);
//Http 响应压缩
builder.Services.AddResponseCompressionSetup();
//Swagger Api 文档
builder.Services.AddSwaggerSetup();
//JWT Token 验证
builder.Services.AddJWTAuthSetup(_configuration);

//MassTransit
builder.Services.AddMassTransitSetup();

// FusionCache
builder.Services.AddFusionCache();

// hangfire  定时任务框架  有界面，更友好~
builder.Services.AddhangfireSetup(_configuration);

//Serilog 日志可视化 LogDashboard日志
builder.Services.AddLogDashboardSetup();

//Dicom影像渲染图片  跨平台
builder.Services.AddDicomSetup();

// 实时应用
builder.Services.AddSignalR();

//MinimalAPI
builder.Services.AddMasaMinimalAPIs(options =>
{
    options.Prefix = "";//自定义前缀 默认是api
    options.Version = ""; //默认是V1
    options.AutoAppendId = false; //路由是否自动附加参数Id  默认是true
    options.PluralizeServiceName = false; //服务名称是否启用复数

    //options.Assemblies = new List<Assembly>() { typeof(UserSiteSurveySubmitedEventConsumer).Assembly };

    options.GetPrefixes = new List<string> { "Get", "Select", "Find" };
    options.PostPrefixes = new List<string> { "Post", "Add", "Create" };
    options.PutPrefixes = new List<string> { "Put", /*"Update"*/ };
    options.DeletePrefixes = new List<string> { "Delete", "Remove" };


    options.DisableAutoMapRoute = false;//可通过配置true禁用全局自动路由映射或者删除此配置以启用全局自动路由映射
});

#endregion

var app = builder.Build();
var env = app.Environment;

#region 配置中间件



app.UseMiddleware<EncryptionRequestMiddleware>();

#region 异常处理  全局业务异常已统一处理了，非业务错误会来到这里  400 -500状态码


//app.UseStatusCodePagesWithReExecute("/Error/{0}");

app.UseStatusCodePages(async context =>
{
    var code = context.HttpContext.Response.StatusCode;
    context.HttpContext.Response.ContentType = "application/json";
    if (code < 500)
    {
        await context.HttpContext.Response.WriteAsync(JsonConvert.SerializeObject(ResponseOutput.NotOk($"Client error, actual request error status code({code})")));
    }
    else
    {
        //ResultFilter 里面的异常并不会到这里
        await context.HttpContext.Response.WriteAsync(JsonConvert.SerializeObject((ResponseOutput.NotOk($"Server error , actual request error status code({code})"))));
    }

});

//app.UseExceptionHandler(o => { });

//这里没生效，原因未知，官方文档也是这种写法，也用了GlobalExceptionHandler 尝试，还是不行，怀疑框架bug
//app.UseExceptionHandler(configure =>
//{
//    configure.Run(async context =>
//    {
//        var exceptionHandlerPathFeature = context.Features.Get<IExceptionHandlerPathFeature>();
//var ex = exceptionHandlerPathFeature?.Error;
//context.Response.ContentType = "application/json";

//if (ex != null)
//{
//    var errorInfo = $"Exception: {ex.Message}[{ex.StackTrace}]" + (ex.InnerException != null ? $" InnerException: {ex.InnerException.Message}[{ex.InnerException.StackTrace}]" : "");

//    await context.Response.WriteAsync(JsonConvert.SerializeObject(ResponseOutput.NotOk($"{ex?.Message}")));

//    Log.Logger.Error(errorInfo);

//}
//    });
//});

#region 暂时废弃

//app.UseMiddleware<MultiDiskStaticFilesMiddleware>();
////限流 中间件
//app.UseIpRateLimiting();
//if (env.IsDevelopment())
//{
//    app.UseDeveloperExceptionPage();
//}
//else
//{
//    //app.UseHsts();
//}

//app.UseIRacisHostStaticFileStore(env);
#endregion

#endregion

app.UseIRacisHostStaticFileStore(env);

//本地化
app.UseLocalization();

app.UseForwardedHeaders();

//响应压缩
app.UseResponseCompression();

//不需要 token 访问的静态文件  wwwroot css, JavaScript, and images don't require authentication.
app.UseStaticFiles();

//LogDashboard
app.UseLogDashboard("/LogDashboard");

//hangfire
app.UseHangfireConfig(env);

// Swagger
SwaggerSetup.Configure(app, env);

//serilog 记录请求的用户信息
app.UseSerilogConfig(env);

app.UseRouting();
app.UseCors(t => t.AllowAnyOrigin().AllowAnyMethod().AllowAnyHeader());
app.UseAuthentication();
app.UseAuthorization();
app.MapControllers();

app.MapHub<UploadHub>("/UploadHub");
app.MapHealthChecks("/health");



#endregion

//Map MinimalAPI  routes
app.MapMasaMinimalAPIs();

// Serilog 
SerilogExtension.AddSerilogSetup(enviromentName, app.Services);

var hangfireJobService = app.Services.GetRequiredService<IIRaCISHangfireJob>();

await hangfireJobService.InitHangfireJobTaskAsync();

try
{
    #region 运行环境 部署平台

    Log.Logger.Warning($"当前环境：{enviromentName}");

    if (RuntimeInformation.IsOSPlatform(OSPlatform.Windows))
    {
        Log.Logger.Warning($"当前部署平台环境：windows");
    }
    else if (RuntimeInformation.IsOSPlatform(OSPlatform.Linux))
    {
        Log.Logger.Warning($"当前部署平台环境：linux");
    }
    else
    {
        Log.Logger.Warning($"当前部署平台环境：OSX  or FreeBSD");
    }

    Log.Logger.Warning($"ContentRootPath：{env.ContentRootPath}");


    string parentDirectory = Path.GetFullPath(Path.Combine(env.ContentRootPath, ".."));


    Log.Logger.Warning($"ContentRootPath——parentDirectory：{parentDirectory}");

    //Log.Logger.Warning($"ContentRootPath——GetParent：{Directory.GetParent(env.ContentRootPath).Parent.FullName}");
    //Log.Logger.Warning($"ContentRootPath——xx：{Path.GetDirectoryName(Path.GetDirectoryName(env.ContentRootPath))}");
    #endregion

    app.Run();
}
catch (Exception e)
{

    Log.Logger.Error(e.InnerException is null ? e.Message + e.StackTrace : e.InnerException?.Message + e.InnerException?.StackTrace);
}



